/* cdw
 * Copyright (C) 2002 Varkonyi Balazs
 * Copyright (C) 2007 - 2009 Kamil Ignacak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include "processwin.h"
#include "gettext.h"
#include "commands.h"
#include "log.h"
#include "options.h"
#include "thread.h" /* PIPE_BUFFER_SIZE */
#include "pipe_regexp.h"
#include "cdrecord_pipe_regexp.h"
#include "cdrecord_interface.h"
#include "utils_string.h"
#include "cdw_disc.h"
#include "cdw_debug.h"


/*
 cdrecord can recognize following disc types (cdrtools-2.01.01, cdrecord/scsi_mmc.c):
"Reserved"
"Non -removable Disk"
"Removable Disk"
"MO Erasable"
"MO Write Once"
"AS-MO"
"CD-ROM"
"CD-R"
"CD-RW"
"DVD-ROM"
"DVD-R sequential recording"
"DVD-RAM"
"DVD-RW restricted overwrite"
"DVD-RW sequential recording"
"DVD-R/DL sequential recording"
"DVD-R/DL layer jump recording"
"DVD-RW/DL"
"DVD+RW"
"DVD+R"
"DDCD-ROM"
"DDCD-R"
"DDCD-RW"
"DVD+RW/DL"
"DVD+R/DL"
"BD-ROM"
"BD-R sequential recording"
"BD-R random recording"
"BD-RE"
"HD DVD-ROM"
"HD DVD-R"
"HD DVD-RAM"
"HD DVD-RW"
"HD DVD-R/DL"
"HD DVD-RW/DL"
"No standard Profile"

some of them are supported by cdw as well. Labels of supported discs are
captured in handle_current_disc_type()
*/



extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1];
extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1];

/* time captured at the beginning of process */
extern time_t time0;


extern struct conf config;
extern cdw_disc_t current_disc;

extern WINDOW *processwin;

extern cdw_task_t *thread_task;

static int previous_percent = 0;


/* functions used for stdout output handling */
static void handle_blanking_time(regex_t *regex, regmatch_t *matches);
static void handle_image_writing(regex_t *regex, regmatch_t *matches);
static void handle_direct_writing(regex_t *regex, regmatch_t *matches);
static void handle_fixating(void);
/* unused: static void handle_is_erasable(regex_t *regex, regmatch_t *matches); */
static void handle_blanking_not_supported(void);
static void handle_drive_max_speed(regex_t *regex, regmatch_t *matches);
static void handle_drive_default_speed(regex_t *regex, regmatch_t *matches);
static void handle_t1_speed(regex_t *regex, regmatch_t *matches);
static void handle_t2_speed(regex_t *regex, regmatch_t *matches);
static void handle_msinfo_sesion_offset(regex_t *regex, regmatch_t *matches);
static void handle_msinfo_session_sectors(regex_t *regex, regmatch_t *matches);
static void handle_current_disc_type(regex_t *regex, regmatch_t *matches);

/* functions used for stderr output handling */
/* unused: static void handle_audio_grab(char *c5); */
static void handle_msinfo_sesion_cannot_get_next(regex_t *regex, regmatch_t *matches);
static void handle_msinfo_no_sesion_offset(regex_t *regex, regmatch_t *matches);
static void handle_device_or_resource_busy(void);
static void handle_cannot_open_device(void);
static void handle_no_such_file_or_directory(void);


static regex_t *regex1 = NULL, *regex2 = NULL, *regex3 = NULL, *regex4 = NULL;
static regex_t *regex5 = NULL, *regex6 = NULL, *regex7 = NULL, *regex8 = NULL;
static regex_t *regex9 = NULL, *regex10 = NULL, *regex11 = NULL, *regex12 = NULL;
static regex_t *regex13 = NULL, *regex14 = NULL;

static regmatch_t *matches1 = NULL, *matches2 = NULL, *matches3 = NULL;
static regmatch_t *matches4 = NULL, *matches5 = NULL, *matches6 = NULL;
static regmatch_t *matches7 = NULL, *matches8 = NULL, *matches9 = NULL;
static regmatch_t *matches10 = NULL, *matches11 = NULL, *matches12 = NULL;
static regmatch_t *matches13 = NULL, *matches14 = NULL;

static regex_t *eregex1 = NULL, *eregex2 = NULL, *eregex3 = NULL;
static regex_t *eregex4 = NULL, *eregex5 = NULL, *eregex6 = NULL;
static regex_t *eregex7 = NULL;

static regmatch_t *ematches1 = NULL, *ematches2 = NULL, *ematches3 = NULL;
static regmatch_t *ematches4 = NULL, *ematches5 = NULL, *ematches6 = NULL;
static regmatch_t *ematches7 = NULL;



/**
 * \brief Allocate memory for regexp data, compile regular expressions (stdout version)
 *
 * Allocate memory for all variables of type regex_t and regmatch_t that
 * are used in processing of data in stdout_pipe_buffer[].
 *
 * Compile all regular expressions that are used in processing of data in
 * stdout_pipe_buffer[].
 *
 * Call regcomp_error_handler() on errors.
 *
 * See also cdrecord_stderr_regexp_prepare()
 */
void cdrecord_stdout_regexp_prepare(void)
{
	int rv;

	/* Blanking time:   27.819s */
	regex1 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex1, "Blanking time:([ ]+)([0-9]*).([0-9]*)s", REG_EXTENDED);
	matches1 = (regmatch_t *) calloc(1, (regex1->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex1, 1);
	}

	/* Direct writing:  */
	/* "Track 01:  190 of  257 MB written (fifo 100%) [buf 100%]  10.3x." */
	regex2 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex2, "Track ([0-9]+): ([ ]*)([0-9]+) of ([ ]*)([0-9]+) MB written ([(])fifo([ ]*)([0-9]+)", REG_EXTENDED);
	matches2 = (regmatch_t *) calloc(1, (regex2->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex2, 2);
	}

	/* Image writing... */
	regex3 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex3, "Track ([0-9]+): ([ ]*)([0-9]+) MB written ([(])fifo([ ]*)([0-9]+)", REG_EXTENDED);
	matches3 = (regmatch_t *) calloc(1, (regex3->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex3, 3);
	}

	/* Fixating... */
	regex4 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex4, "Fixating...", REG_EXTENDED);
	matches4 = (regmatch_t *) calloc(1, (regex4->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex4, 4);
	}

	/* Is erasable... */
	regex5 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex5, "Is erasable", REG_EXTENDED);
	matches5 = (regmatch_t *) calloc(1, (regex5->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex5, 5);
	}

	/* this media does not support blanking */
	regex6 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex6, "this media does not support blanking", REG_EXTENDED);
	matches6 = (regmatch_t *) calloc(1, (regex6->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex6, 6);
	}

	/* "Drive default speed: xy" */
	regex7 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex7, "Drive default speed([ ]*):([ ]*)([0-9]*)", REG_EXTENDED);
	matches7 = (regmatch_t *) calloc(1, (regex7->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex7, 7);
	}

	/* "Drive max speed    : xy" */
	regex8 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex8, "Drive max speed([ ]*):([ ]*)([0-9]*)", REG_EXTENDED);
	matches8 = (regmatch_t *) calloc(1, (regex8->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex8, 8);
	}

	/* "1T speed low:  4 1T speed high: 10" */
	regex9 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex9, "1T speed low:([ ]*)([0-9]*)([ ]*)1T speed high:([ ]*)([0-9]*)", REG_EXTENDED);
	matches9 = (regmatch_t *) calloc(1, (regex9->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex9, 9);
	}

	/* "2T speed low:  4 2T speed high:  0 */ /* (reserved val  6)" */
	regex10 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex10, "2T speed low:([ ]*)([0-9]*)([ ]*)2T speed high:([ ]*)([0-9]*)", REG_EXTENDED);
	matches10 = (regmatch_t *) calloc(1, (regex10->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex10, 10);
	}

	/* "session offset:" - printed for appendable discs, but also for closed discs; keep colon! */
	regex11 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex11, "(session offset:)", REG_EXTENDED);
	matches11 = (regmatch_t *) calloc(1, (regex11->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex11, 11);
	}

	/* "Current: 0x000A (CD-RW)" - wodim reports type of disc currently in drive (for both CD and DVD) */
	regex12 = (regex_t *) calloc(1, sizeof(regex_t));
	/* '0x' is added to make sure that this will be recognized as long format */
	rv = regcomp(regex12, "Current:([ ]*)([0]*)([x]*)([0-9a-fA-F]*)([ ]*)([(]*)([A-Z+\-]+)([)]*)", REG_EXTENDED);
	matches12 = (regmatch_t *) calloc(1, (regex12->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex12, 12);
	}

	/* "Current: CD-RW" - cdrecord reports type of disc currently in drive (for both CD and DVD) */
	regex13 = (regex_t *) calloc(1, sizeof(regex_t));
	/* ([CDVRWOM+\-]+) on purpose don't include '0x' to distinguish long format from short */
	rv = regcomp(regex13, "Current:([ ]*)([CDVRWOM+\-]+)", REG_EXTENDED);
	matches13 = (regmatch_t *) calloc(1, (regex13->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex13, 13);
	}

	/* "122176,129604" - information about last_sess_start and
	    next_sess_start sectors, comma is significant; this is result
	    of running cdrecord -msinfo */
	regex14 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex14, "([0-9]+),([0-9]+)", REG_EXTENDED);
	matches14 = (regmatch_t *) calloc(1, (regex14->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex14, 14);
	}


	return;
}





/**
 * \brief Match data in stdout_pipe_buffer[] agains regular expressions, call handler function
 *
 * This function calls regexec() for all regular expressions to compare them
 * against data in stdout_pipe_buffer[]. If there will be any match, the
 * function will call proper haldler function to further process data from
 * stdout_pipe_buffer[]
 */
void cdrecord_stdout_regexp_execute(void)
{
	stdout_pipe_buffer[PIPE_BUFFER_SIZE - 1] = '\0';

	/* Blanking time:   27.819s - printed after successfull blanking of CD */
	int rv1 = regexec(regex1, stdout_pipe_buffer, regex1->re_nsub + 1, matches1, 0);
	if (rv1 == 0) {
		cdw_sdm("calling handle_blanking_time()\n");
		handle_blanking_time(regex1, matches1);
	}

	/* Direct writing */
	int rv2 = regexec(regex2, stdout_pipe_buffer, regex2->re_nsub + 1, matches2, 0);
	if (rv2 == 0) {
		cdw_sdm("calling handle_direct_writing()\n");
		handle_direct_writing(regex2, matches2);
	}

	/* Image writing... */
	int rv3 = regexec(regex3, stdout_pipe_buffer, regex3->re_nsub + 1, matches3, 0);
	if (rv3 == 0) {
		cdw_sdm("calling handle_image_writing()\n");
		handle_image_writing(regex3, matches3);
	}

	/* Fixating */
	int rv4 = regexec(regex4, stdout_pipe_buffer, regex4->re_nsub + 1, matches4, 0);
	if ((rv4 == 0) && (rv2 != 0)) {
		cdw_sdm("calling handle_fixating()\n");
		handle_fixating();
	}

	/* Is erasable */
	int rv5 = regexec(regex5, stdout_pipe_buffer, regex5->re_nsub + 1, matches5, 0);
	if (rv5 == 0) {
		/* 'erasability' status is handled in handle_current_disc_type() */
		/* handle_is_erasable(regex5, matches5); */
	}

	/* this media does not support blanking */
	int rv6 = regexec(regex6, stdout_pipe_buffer, regex6->re_nsub + 1, matches6, 0);
	if (rv6 == 0) {
		cdw_sdm("calling handle_blanking_not_supported()\n");
		handle_blanking_not_supported();
	}

	/* "Drive default speed: xy" */
	int rv7 = regexec(regex7, stdout_pipe_buffer, regex7->re_nsub + 1, matches7, 0);
	if (rv7 == 0) {
		cdw_sdm("calling handle_drive_max_speed()\n");
		handle_drive_max_speed(regex7, matches7);
	}

	/* "Drive max speed    : xy" */
	int rv8 = regexec(regex8, stdout_pipe_buffer, regex8->re_nsub + 1, matches8, 0);
	if (rv8 == 0) {
		cdw_sdm("calling handle_drive_default_speed()\n");
		handle_drive_default_speed(regex8, matches8);
	}

	/* "1T speed low:  4 1T speed high: 10" */
	int rv9 = regexec(regex9, stdout_pipe_buffer, regex9->re_nsub + 1, matches9, 0);
	if (rv9 == 0) {
		cdw_sdm("calling handle_t1_speed()\n");
		handle_t1_speed(regex9, matches9);
	}

	/* "2T speed low:  4 2T speed high:  0 */ /* (reserved val  6)" */
	int rv10 = regexec(regex10, stdout_pipe_buffer, regex10->re_nsub + 1, matches10, 0);
	if (rv10 == 0) {
		cdw_sdm("calling handle_t2_speed()\n");
		handle_t2_speed(regex10, matches10);
	}

	/* "session offset:" */
	int rv11 = regexec(regex11, stdout_pipe_buffer, regex11->re_nsub + 1, matches11, 0);
	if (rv11 == 0) {
		cdw_sdm("calling handle_msinfo_sesion_offset()\n");
		handle_msinfo_sesion_offset(regex11, matches11);
	}

	/* "Current: 0x000A (CD-RW)" */
	int rv12 = regexec(regex12, stdout_pipe_buffer, regex12->re_nsub + 1, matches12, 0);
	if (rv12 == 0) {
		cdw_sdm("calling handle_current_disc_type(long version)\n");
		handle_current_disc_type(regex12, matches12);
	}

	/* "Current: CD-RW" */
	int rv13 = regexec(regex13, stdout_pipe_buffer, regex13->re_nsub + 1, matches13, 0);
	if (rv13 == 0) {
		cdw_sdm("calling handle_current_disc_type(short version)\n");
		handle_current_disc_type(regex13, matches13);
	}

	/* "122176,129604" */
	int rv14 = regexec(regex14, stdout_pipe_buffer, regex14->re_nsub + 1, matches14, 0);
	if (rv14 == 0) {
		cdw_sdm("calling handle_msinfo_session_sectors()\n");
		handle_msinfo_session_sectors(regex14, matches14);
	}


	return;
}





/**
 * \brief Free all memory used by regular expression variables (stdout version)
 *
 * Call regfree() and free() all variables of type regex_t and regmatch_t that
 * were used in processing of data in stdout_pipe_buffer[].
 *
 * See also cdrecord_stderr_regexp_destroy()
 */
void cdrecord_stdout_regexp_destroy(void)
{
	regfree(regex1);
	regfree(regex2);
	regfree(regex3);
	regfree(regex4);
	regfree(regex5);
	regfree(regex6);
	regfree(regex7);
	regfree(regex8);
	regfree(regex9);
	regfree(regex10);
	regfree(regex11);
	regfree(regex12);
	regfree(regex13);
	regfree(regex14);

	free(regex1);
	free(regex2);
	free(regex3);
	free(regex4);
	free(regex5);
	free(regex6);
	free(regex7);
	free(regex8);
	free(regex9);
	free(regex10);
	free(regex11);
	free(regex12);
	free(regex13);
	free(regex14);

	free(matches1);
	free(matches2);
	free(matches3);
	free(matches4);
	free(matches5);
	free(matches6);
	free(matches7);
	free(matches8);
	free(matches9);
	free(matches10);
	free(matches11);
	free(matches12);
	free(matches13);
	free(matches14);

	return;
}





/**
 * \brief Allocate memory for regexp data, compile regular expressions (stderr version)
 *
 * Allocate memory for all variables of type regex_t and regmatch_t that
 * are used in processing of data in stderr_pipe_buffer[].
 *
 * Compile all regular expressions that are used in processing of data in
 * stderr_pipe_buffer[].
 *
 * Call regcomp_error_handler() on errors.
 *
 * See also cdrecord_stdout_regexp_prepare()
 */
void cdrecord_stderr_regexp_prepare(void)
{
	int rv = 0;

	/* Device or resource busy - system message when cdrecord tries to access busy cd. */
	eregex1 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex1, "Device or resource busy", REG_NOSUB);
	ematches1 = (regmatch_t *) calloc(1, (eregex1->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex1, 21);
	}

	/* cannot write medium - incompatible format */
	eregex2 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex2, "cannot write medium - incompatible format", REG_EXTENDED);
	ematches2 = (regmatch_t *) calloc(1, (eregex2->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex2, 22);
	}

	/* Cannot blank disk */
	eregex3 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex3, "Cannot blank disk", REG_EXTENDED);
	ematches3 = (regmatch_t *) calloc(1, (eregex3->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex3, 23);
	}

	/* "Cannot get next writable address" - printed for 'complete' discs */
	eregex4 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex4, "(Cannot get next writable address)", REG_EXTENDED);
	ematches4 = (regmatch_t *) calloc(1, (eregex4->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex4, 24);
	}

	/* "Cannot read session offset" - printed for blank discs */
	eregex5 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex5, "(Cannot read session offset)", REG_EXTENDED);
	ematches5 = (regmatch_t *) calloc(1, (eregex5->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex5, 25);
	}

	/* "cdrecord: Permission denied. Cannot open '/dev/sg0'. Cannot open or use SCSI driver." */
	/* TODO: test this for wodim too */
	eregex6 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex6, "(cdrecord|wodim): Permission denied. Cannot open", REG_EXTENDED);
	ematches6 = (regmatch_t *) calloc(1, (eregex6->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex6, 26);
	}

	/* "cdrecord: No such file or directory. Cannot open 'some'." */
	/* this is similar to "No such file or directory. Invalid node"
	   from mkisofs - propbably incorrect entry in "other cdrecord options" */
	/* TODO: test this for wodim too */
	eregex7 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex7, "(cdrecord|wodim): No such file or directory. Cannot open", REG_EXTENDED);
	ematches7 = (regmatch_t *) calloc(1, (eregex7->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex7, 27);
	}

	return;
}





/**
 * \brief Match data in stderr_pipe_buffer[] agains regular expressions, call handler function
 *
 * This function calls regexec() for all regular expressions to compare them
 * against data in stderr_pipe_buffer[]. If there will be any match, the
 * function will call proper haldler function to further process data from
 * stderr_pipe_buffer[]
 */
void cdrecord_stderr_regexp_execute(void)
{
	stderr_pipe_buffer[PIPE_BUFFER_SIZE - 1] = '\0';

	/* Device or resource busy. */
	int rv1 = regexec(eregex1, stderr_pipe_buffer, eregex1->re_nsub + 1, ematches1, 0);
	if (rv1 == 0) {
		cdw_sdm("calling handle_device_or_resource_busy()\n");
		handle_device_or_resource_busy();
	}

	/* cannot write medium - incompatible format */
	int rv2 = regexec(eregex2, stderr_pipe_buffer, eregex2->re_nsub + 1, ematches2, 0); /* k */
	if (rv2 == 0) {
		cdw_sdm("calling handle_blanking_not_supported()\n");
		/* media is not erasable */
		handle_blanking_not_supported();
	}

	/* Cannot blank disk */
	int rv3 = regexec(eregex3, stderr_pipe_buffer, eregex3->re_nsub + 1, ematches3, 0); /* k */
	if (rv3 == 0) {
		cdw_sdm("calling handle_blanking_not_supported()\n");
		/* media is not erasable */
		handle_blanking_not_supported();
	}

	/* "Cannot get next writable address" */
	int rv4 = regexec(eregex4, stderr_pipe_buffer, eregex4->re_nsub + 1, ematches4, 0);
	if (rv4 == 0) {
		cdw_sdm("calling handle_msinfo_sesion_cannot_get_next()\n");
		handle_msinfo_sesion_cannot_get_next(eregex4, ematches4);
	}

	/* "Cannot read session offset" */
	int rv5 = regexec(eregex5, stderr_pipe_buffer, eregex5->re_nsub + 1, ematches5, 0);
	if (rv5 == 0) {
		cdw_sdm("calling handle_msinfo_no_sesion_offset()\n");
		handle_msinfo_no_sesion_offset(eregex5, ematches5);
	}

	/* "cdrecord: Permission denied. Cannot open '/dev/sg0'. Cannot open or use SCSI driver." */
	int rv6 = regexec(eregex6, stderr_pipe_buffer, eregex6->re_nsub + 1, ematches6, 0);
	if (rv6 == 0) {
		cdw_sdm("calling handle_cannot_open_device()\n");
		handle_cannot_open_device();
	}

	/* "cdrecord: No such file or directory. Cannot open 'some'." */
	int rv7 = regexec(eregex7, stderr_pipe_buffer, eregex7->re_nsub + 1, ematches7, 0);
	if (rv7 == 0) {
		cdw_sdm("calling handle_no_such_file_or_directory()\n");
		handle_no_such_file_or_directory();
	}



	/* char c1[25], c2[25], c3[25], c4[25], c5[25], tmp[25]; */

	/* Cdda2wav process */
	/* ??/??/??/???????   0% */
	/*
	if (sscanf(stderr_pipe_buffer, "%s %s %s %s %s %s", c1, c2, c3, c4, c5, tmp) == 5) {
		handle_audio_grab(c5);
	}
	*/


	return;
}





/**
 * \brief Free all memory used by regular expression variables (stderr version)
 *
 * Call regfree() and free() all variables of type regex_t and regmatch_t that
 * were used in processing of data in stderr_pipe_buffer[].
 *
 * See also cdrecord_stdout_regexp_destroy()
 */
void cdrecord_stderr_regexp_destroy(void)
{
	regfree(eregex1);
	regfree(eregex2);
	regfree(eregex3);
	regfree(eregex4);
	regfree(eregex5);
	regfree(eregex6);
	regfree(eregex7);

	free(eregex1);
	free(eregex2);
	free(eregex3);
	free(eregex4);
	free(eregex5);
	free(eregex6);
	free(eregex7);

	free(ematches1);
	free(ematches2);
	free(ematches3);
	free(ematches4);
	free(ematches5);
	free(ematches6);
	free(ematches7);

	return;
}





void handle_image_writing(regex_t *regex, regmatch_t *matches)
{
	/*          1         2     3                  4        5      6      */
	/* "Track ([0-9]+): ([ ]*)([0-9]+) MB written ([(])fifo([ ]*)([0-9]+)"*/

	/* data to collect */
	int fifo = 0;
	int done_size = 0;
	int tracknum = 0;

	/* data to calculate */
	char text_info2_string[PROCESSWIN_MAX_TEXT_LEN];
	char current_value_string[PROCESSWIN_MAX_TEXT_LEN];

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
 		if (i == 1) {
			tracknum = atoi((char *) &submatch);
			/* debug message */
			/* fprintf(stderr, "%s(): tracknum = \"%d\"\n", __func__, tracknum); */
		} else if (i == 3) {
			done_size = atoi((char *) &submatch);
			/* debug message */
			/* fprintf(stderr, "%s(): done_size = \"%d\"\n", __func__, done_size); */
		} else if (i == 6) {
			fifo = atoi((char *) &submatch);
			/* debug message */
			/* fprintf(stderr, "%s(): fifo = \"%d\"\n", __func__, fifo); */
		} else {
			/* debug message */
			/* fprintf(stderr, "%s(): i = %d, submatch = \"%s\"\n", __func__, i, submatch); */
		}
	}

/*
	if (tracknum != lasttrack) {
	mvwprintw(processwin, 5, 5, "");
	processwin_draw_progress_bar(19, 2, 2, ' ');
}

	lasttrack = tracknum;

	if (total_size < new_total) {
	total_size = new_total;
}
*/
	/* 2TRANS: this is label displayed in progress window,
	   Track is cd data track, %d is its number */
	sprintf(text_info2_string, _("Track: %d"), tracknum);
	/* 2TRANS: this is label displayed in progress window,
	   %d is amount of data already written to CD */
	sprintf(current_value_string, _("%d MB"), done_size);


	/* display data */
	conditional_processwin_display_progress(done_size, 0, current_value_string);
	processwin_display_text_info(2, text_info2_string);
	processwin_display_fifo(fifo);
	wrefresh(processwin);

	return;
}





void handle_direct_writing(regex_t *regex, regmatch_t *matches)
{
	/*           1        2      3          4      5                  6        7      8      */
	/* "Track ([0-9]+): ([ ]*)([0-9]+) of ([ ]*)([0-9]+) MB written ([(])fifo([ ]*)([0-9]+)" */

	/* data to collect */
	int fifo = 0;
	int total_size = 0;
	int done_size = 0;
	int tracknum = 0;

	/* data to calculate */
	char text_info2_string[PROCESSWIN_MAX_TEXT_LEN];
	char eta_string[PROCESSWIN_MAX_TEXT_LEN];
	char current_value_string[PROCESSWIN_MAX_TEXT_LEN];

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		if (i == 1) {
			tracknum = atoi((char *) &submatch);
			cdw_sdm("tracknum = %d\n", tracknum);
		} else if (i == 3) {
			done_size = atoi((char *) &submatch);
			cdw_sdm("done_size = %d\n", done_size);
		} else if (i == 5) {
			total_size = atoi((char *) &submatch);
			cdw_sdm("total_size = %d\n", total_size);
		} else if (i == 8) {
			fifo = atoi((char *) &submatch);
			cdw_sdm("fifo = %d\n", fifo);
		} else {
			cdw_sdm("i = %d, submatch = %s\n", i, submatch);
		}
	}

/*
	if (tracknum != lasttrack) {
	mvwprintw(processwin, 5, 5, "");
	processwin_draw_progress_bar(19, 2, 2, ' ');
}
	lasttrack = tracknum;

	if (total_size < new_total) {
	total_size = new_total;
}
*/

	/* compute data */

	int percent = (done_size * 100) / total_size;
	if ((percent >= 0) && (percent <= 100)) {
		cdw_sdm("percent = %d\n", percent);

		if (previous_percent != percent) {
			cdw_sdm("previous_percent = %d, percent = %d\n", previous_percent, percent);

			int eta = (int) ((difftime(time(NULL), time0) * (total_size - done_size)) / done_size);
			eta_calculations(eta_string, eta);
			previous_percent = percent;
			processwin_display_eta(eta_string);

			cdw_sdm("eta = %d, eta_string = %s\n", eta, eta_string);
		}
	}

	/* 2TRANS: this is label displayed in progress window, first %d is amount
	of data already written to CD, second %d is total amount of data to write */
	sprintf(current_value_string, _("%d/%d MB"), done_size, total_size);
	/* 2TRANS: this is label displayed in progress window,
	   Track is cd data track, %d is its number */
	sprintf(text_info2_string, _("Track: %d"), tracknum);
	processwin_display_text_info(2, text_info2_string);

	/* display data */

	conditional_processwin_display_progress(done_size, total_size, current_value_string);
	processwin_display_fifo(fifo);
	wrefresh(processwin);

	return;
}





void handle_fixating(void)
{
	/* 2TRANS: this is message displayed in process window,
	   meaning that closing session is in progress */
	processwin_display_text_info(1, _("Fixating..."));

	/* this is unnecessary - writing process always (?) displays 100%,
	so it will be displayed in processwin as well */
	/* processwin_display_progress(100, 100, ""); */

	wrefresh(processwin);

	return;
}





#if 0
/* currently unused, disabling to supress compiler warnings */
void handle_audio_grab(char *c5)
{
	int perc = atoi(c5);
	char percent[30];

	if ((thread_task->id == TASK_READ && thread_task->tool == TOOL_CDRECORD) && (perc <= 100)) {
		if (previous_percent != perc) {
			char eta_string[25];
			int eta = (int) ((difftime(time(NULL), time0) * (100 - perc)) / perc);
			eta_calculations(eta_string, eta);
			previous_percent = perc;
		}
		mvwprintw(processwin, 5, 5, "");
		processwin_draw_progress_bar((perc / 5), 5, 2, ACS_BLOCK);

		/* 2TRANS: this string is currently not used; %d%% is amount of audio
		data already read from cd (in percents) */
		sprintf(percent, _("  %d%% done  "), perc);
		mvwprintw(processwin, 3, 15 - (strlen(percent) / 2), "%s", percent);
		/* mvwprintw(processwin, 5, 5, "");
		mvwprintw(processwin, 9, 29, "");
		mvwprintw(processwin, 7, 5, "T: %d P: %d L: %d   ", track, perc, lasttrack); */
		wrefresh(processwin);
		/* fprintf(log, "\nT: %d P: %d L: %d\n", track, perc, lasttrack); */
		/* fflush(logfi); */
		stderr_pipe_buffer[0] = '\0';
		/* track=perc;
		sleep(1); */
	}

	return;
}
#endif





#if 0
/* currently unused, disabling to supress compiler warnings */
void handle_is_erasable(regex_t *regex, regmatch_t *matches)
{
	current_disc.erasable = CDW_DISC_ERASABLE_YES;

	return;
}
#endif





/**
 * When attempting to blank media cdrecord may say this:
 *
 * wodim: Cannot blank disk, aborting.
 * wodim: Some drives do not support all blank types.
 * wodim: Try again with wodim blank=all.
 *
 * This function is called when cdw catches "Cannot blank disk". The function
 * sets current_disc.erasable to CDW_DISC_ERASABLE_NO, so that higher level
 * code will know that blanking failed and perhaps blanking with mode 'all' is
 * required.
 */
void handle_blanking_not_supported(void)
{

	current_disc.erasable = CDW_DISC_ERASABLE_NO;

	return;
}





void handle_device_or_resource_busy(void)
{
	/* 2TRANS: this is message in progress window, meaning that
	   subprocess cannot access CD drive, because the drive is busy */
	processwin_display_text_info(1, _("Device or resource busy."));
	/* 2TRANS: this is message in progress window,
	   possible explanation for previous message */
	processwin_display_text_info(2, _("Make sure that device is unmounted."));
	wrefresh(processwin);

	/* FIXME: cdrecord will try for a few seconds to access cdrom and
	   then exits. During the tries cdw will display this information, and
	   then will try to read cd info once again, ant then will (or will
	   not) display log with unsuccessful tries. Display a message
	   instead of a log */

	return;
}





void handle_drive_default_speed(regex_t *regex, regmatch_t *matches)
{
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 3) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			current_disc.drive.default_speed = atoi((char *) &submatch);
		}
		/* only 3 submatches defined */
	}

	return;
}





void handle_drive_max_speed(regex_t *regex, regmatch_t *matches)
{
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 3) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			current_disc.drive.max_speed = atoi((char *) &submatch);
		}
		/* only 3 submatches defined */
	}

	return;
}





void handle_t1_speed(regex_t *regex, regmatch_t *matches)
{
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			current_disc.cdrecord.atip_t1_low_speed = atoi((char *) &submatch);
		} else if (i == 3) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 4) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			current_disc.cdrecord.atip_t1_high_speed = atoi((char *) &submatch);
		} else if (i == 5) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		}
		/* only 5 submatches defined */
	}

	return;
}





void handle_t2_speed(regex_t *regex, regmatch_t *matches)
{
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			current_disc.cdrecord.atip_t2_low_speed = atoi((char *) &submatch);
		} else if (i == 3) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 4) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 5) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			current_disc.cdrecord.atip_t2_high_speed = atoi((char *) &submatch);
		}
		/* only 5 submatches defined */
	}

	return;
}





void handle_msinfo_sesion_offset(regex_t *regex, regmatch_t *matches)
{
	/* debug message */
	/* fprintf(stderr, "----%s(): START\n", __func__); */
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */

		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		}
		/* only 1 submatch defined */
	}

	return;
}





void handle_msinfo_sesion_cannot_get_next(regex_t *regex, regmatch_t *matches)
{
	/* debug message */
	/* fprintf(stderr, "%s() called\n", __func__); */
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stderr_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */

		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* can't get multisession info for next
			   session - to be interpreted by highger-level code */
			current_disc.last_sess_start = -1;
			current_disc.next_sess_start = -1;

		}
		/* only 1 submatch defined */
	}
	/* fprintf(stderr, "%s() current_disc.last_sess_start=%ld, current_disc.next_sess_start=%ld\n", __func__, current_disc.last_sess_start, current_disc.next_sess_start); */

	return;

}





void handle_msinfo_no_sesion_offset(regex_t *regex, regmatch_t *matches)
{
	/* debug message */
	/* fprintf(stderr, "%s() called\n", __func__); */
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stderr_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */

		if (i == 1) {
			/* can't get multisession info about any existing
			   session (disc is blank?) - to be interpreted
			   by highger-level code */
			current_disc.last_sess_start = 0;
			current_disc.next_sess_start = 0;

			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		}
		/* only 1 submatch defined */
	}

	return;
}





/**
 * \brief Read current disc type (vesion capturing short and long format of information)
 *
 * 'cdrecord -atip -v' prints information about disc type. Depending on
 * version of cdrecord, the format can be:
 * \li short: "Current: CD-RW"
 * \li long: "Current: 0x000A (CD-RW)"
 *
 * where 'CD-RW' can be replaced by other disc types. List of disc types
 * recognized by cdrecord is on top of this file. Some of the types are
 * also supported by cdw. For discs supported by cdw proper fields in
 * current_disc will be set to corresponding values. Otherwise these
 * fields will be set to 'unknown' values.
 */
void handle_current_disc_type(regex_t *regex, regmatch_t *matches)
{
	/* long format: */
        /*            1     2     3         4         5      6       7       8     */
	/* "Current:([ ]*)([0]*)([x]*)([0-9a-fA-F]*)([ ]*)([(]*)([A-Z+\-]+)([)]*)" */
	/* "Current: 0x000A (CD-RW)" */

	/* short format */
	/*            1         2          */
	/* "Current:([ ]*)([CDVRWOM+\-]+)" */
	/* "Current: CD-RW" */

	/* function distinguishes between these two formats checking submatch2 */

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch%d: \"%s\"\n", __func__, i, submatch); */

		/* information about disc type in short format is stored in
		   submatch2 and in long format in submatch7; submatch2 may
		   also store "0" in long format; submatch7 does not exist
		   for short format */
		if (i == 2 || i == 7) {
			if (!strcmp(submatch, "0")) {
				continue; /* submatch 2, but format is long */
			}

			/* at this point, regardless of format,
			   submatch stores disc type */

			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			if (!strcmp(submatch, "CD-RW")) {
				current_disc.disc_type = CDW_CD_RW;
			} else if (!strcmp(submatch, "CD-R")) {
				current_disc.disc_type = CDW_CD_R;
			} else if (!strcmp(submatch, "CD-ROM")) {
				if (current_disc.cdio_disc.mode == CDIO_DISC_MODE_CD_DA) {
					current_disc.disc_type = CDW_CD_AUDIO;
				} else {
					current_disc.disc_type = CDW_CD_ROM;
				}
			} else {
				current_disc.disc_type = CDW_DISC_UNKNOWN;
			}
		} else {
			cdw_sdm ("submatch%d: \"%s\"\n", i, submatch);
		}
	}

	return;
}





/**
 * \brief Get multisession info printed by cdrecord when called with -msinfo option
 *
 * This function is called if cdrecord prints addresses of:
 * \li beginning of last written session
 * \li beginning of next session
 *
 * that may be written in following format: X,Y
 * Capturing such information means that this disc is appendable (because
 * the second address is valid). Therefore this is the only reasonable place
 * to set current_disc.disc_write_status = CDW_DISC_W_APPENDABLE in this file.
 * If disc was blank then we would have captured "Cannot read session offset";
 * if the disc was closed, we would capture "Cannot get next writable address".
 */
void handle_msinfo_session_sectors(regex_t *regex, regmatch_t *matches)
{
	/* debug message */
	/* fprintf(stderr, "%s() called\n", __func__); */

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */

		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			current_disc.last_sess_start = atol(submatch);
			/* fprintf(stderr, "%s(): submatch%d -> %ld\n", __func__, i, current_disc.last_sess_start); */
		} else if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			current_disc.next_sess_start = atol(submatch);
			/* fprintf(stderr, "%s(): submatch%d -> %ld\n", __func__, i, current_disc.next_sess_start); */
		} else {
			;
		}
	}
	/* This function is called when two numbers in format XXX,YYY are
	   captured - such numbers are printed by cdrecord for appendable
	   CD. If the disc was not appendable, cdrecord output would be
	   captured by void handle_msinfo_sesion_cannot_get_next() */

	/* cdrecord doesn't print msinfo for empty disc (it would be 0,0),
	   so in this situation disc is not empty; cdrecord informs about
	   empty CD-R/CD-RW disc by writing that it cannot get session
	   offset */

	/* debug messages */
	/*
	fprintf(stderr, "%s(): setting disc status to CDW_DISC_W_APPENDABLE\n", __func__);
	fprintf(stderr, "%s() current_disc.last_sess_start=%ld, current_disc.next_sess_start=%ld\n", __func__, current_disc.last_sess_start, current_disc.next_sess_start);
	*/

	return;
}





/**
 * \brief Get information about duration of blanking process
 *
 * Format of such information is following:
 *"Blanking time:   27.819s"
 *
 * If cdrecord/wodim prints this information then blanking was successfull
 */
void handle_blanking_time(regex_t *regex, regmatch_t *matches)
{
	/*                  1      2        3       */
	/* "Blanking time:([ ]+)([0-9]*).([0-9]*)s" */

	int seconds = 0;
	int miliseconds = 0;

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		if (i == 1) { /* spaces */
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 2) { /* 'seconds' part of time of blanking */
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			seconds = atoi(submatch);
		} else if (i == 3) { /* 'miliseconds' part of time of blanking */
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			miliseconds = atoi(submatch);
		} else {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		}
	}

	/* in case of correct blanking time of blanking will always be
	   more than 0 seconds, so I don't check 'if (seconds == 0)' */

	if (miliseconds > 0) {
		thread_task->blanking.blanking_time = seconds + 1;
	} else {
		thread_task->blanking.blanking_time = seconds;
	}

	return;
}





void handle_cannot_open_device(void)
{
	thread_task->tool_status |= TOOL_STATUS_CDRECORD_CANNOT_OPEN_DEVICE;

	return;
}





void handle_no_such_file_or_directory(void)
{
	cdw_sdm("non-masked tool_status = 0x%04x\n", thread_task->tool_status);
	thread_task->tool_status |= TOOL_STATUS_CDRECORD_NO_SUCH_FILE_OR_DIR;

	return;
}


