#ifndef CDW_CDIO_H__
#define CDW_CDIO_H__

#include <ncursesw/ncurses.h>
#include <cdio/cdio.h>
#include "main.h"

typedef enum cdw_track_format {
	CDW_RED_BOOK_AUDIO,             /* all 2352 bytes are useful data */
	CDW_YELLOW_BOOK_MODE1,          /* sync + header + 2048 bytes of useful data + some error magic */
	CDW_YELLOW_BOOK_MODE2_FORMLESS, /* sync + header + subheader + 2336 bytes of useful data + some error magic */
	CDW_YELLOW_BOOK_MODE2_FORM1,    /* sync + header + subheader + 2048 bytes of useful data + some error magic */
	CDW_YELLOW_BOOK_MODE2_FORM2,    /* sync + header + subheader + 2324 bytes of useful data + some error magic */
	CDW_MIXED,                      /* I'm puzzled, but ioctl() returns CDS_MIXED, which means data + audio */
	CDW_DVD,                        /* DVD disc- all DVD discs have the same size of useful data in sector (2048) */
	CDW_BLACK_BOOK_UNKNOWN          /* unrecognized disc type */
} cdw_track_format_t;



/* CD, DVD or UNKNOWN - sometimes knowing this is enough */
typedef enum cdw_simple_disc_type {
	DISC_NONE,
	CD,
	DVD,
	UNKNOWN
} cdw_simple_disc_type_t;





typedef struct cdw_track {
	track_format_t cdio_format;    /* AUDIO, DATA, XA, CDI, PSX, ERROR - this is track format provided by libcdio */
	cdw_track_format_t cdw_format; /* this is track format defined in cdw, set by cdw code - it has more direct relation to size of useful data in sector */
	lsn_t first_sector;
	lsn_t last_sector;
	lsn_t total_sectors;  /* (last_sector - first_sector + 1) this value may include padding zeros - you may not want to read this */
	size_t sectors_to_read; /* real number of sectors with real data, may be smaller than total number of sectors in track */
	size_t sector_size;   /* size of sectors in given track */
} cdw_track_t;


typedef struct cdw_cdio_disc {
	CdIo_t *p_cdio;       /* libcdio data structure */
	discmode_t mode;      /* disc mode as defined in cdio/disc.h */

	/* int ioctl_mode; not used anymore (it was called mode2 some time ago) */

	bool open;            /* is the dics opened already */
	bool blank;

	lsn_t total_sectors;  /* total amount of sectors of all kinds */

	track_t first_track;  /* first track on disc, most probably always equals to 1 */
	track_t last_track;   /* last track on disc */
	track_t total_tracks; /* total amount of tracks of all kinds */

	track_t unknown_tracks; /* total amount of tracks of unknown type */
	track_t fuzzy_tracks; /* total amount of tracks of type that can't be set with 100% certainity */

	cdw_track_t tracks[100]; /* CD disc has no more than 99 tracks, track numbering by cdio is from 1 */
} cdw_cdio_disc_t;


/*
typedef driver_return_code_t (*sector_reader_fun)(const CdIo_t *p_cdio, void *p_buf, lsn_t i_lsn, bool b_form2, uint32_t i_blocks);
*/

cdw_rv_t cdw_cdio_disc_open(char *device);
void cdw_cdio_disc_close(void);
cdw_rv_t cdw_cdio_open_get_meta_close(char *device);
cdw_rv_t cdw_cdio_get_disc_meta_information(void);
void cdw_cdio_disc_reset(void);

bool cdw_cdio_disc_supported_for_cdw_ripping(void);

cdw_rv_t cdw_cdio_copy_tracks_to_file(int output_image_file);
cdw_rv_t cdw_cdio_copy_sectors_to_file(track_t track, int output_image_file);

#endif /* CDW_CDIO_H__ */

