#ifndef CDW_WIDGETS_H__
#define CDW_WIDGETS_H__

#include <ncursesw/ncurses.h>
#include <ncursesw/menu.h>
#include "main.h" /* cdw_rv_t */


/**
 * \brief Type of simple dialog window
 *
 * This value defines number and types of buttons.
 */
enum {
	DIALOG_OK = 0,
	DIALOG_OK_CANCEL = 3,
	DIALOG_YES_NO_CANCEL = 4
};


typedef struct {
	WINDOW *parent;
	WINDOW *strip;

	int begin_x;
	int begin_y;

	char *label;
	int label_len;

	int colors;
} CDW_BUTTON;




typedef struct {
	WINDOW *parent;
	int parent_x;
	int parent_y;

	int width;

	const char **labels;
	int nvalues;

	ITEM **fitems;
	MENU *fmenu;

	int current_item_ind;

	int colors;

	WINDOW *menu_window;
	WINDOW *menu_window_sub;
	int begin_x;
	int begin_y;

} CDW_DROPDOWN;


enum {
	CDW_ALIGN_LEFT,
	CDW_ALIGN_RIGHT,
	CDW_ALIGN_CENTER
};


/* see explanation for this in comment for one_line_input()
and inputbox() in cdw_widgets.c */
#define CDW_HASH 2
#define CDW_FIELD_TYPE_INTEGER 4
#define CDW_FIELD_TYPE_NUMERIC 8


cdw_rv_t one_line_input(WINDOW *window, char **buffer, int mode, size_t limit);
cdw_rv_t nice_box(WINDOW *window, const char *top_string, const char *bottom_string);
int dialogbox(const char *title, const char *message, int dialog_type, int cdw_colors);
cdw_rv_t inputbox(const char *title, const char *label, char **buffer, int mode, int limit);


void put_ckbox(WINDOW *win, int y, int x);

CDW_BUTTON *cdw_button_new(WINDOW *parent, int begin_y, int begin_x, const char *label, int colors);
void cdw_button_destroy(CDW_BUTTON *button);
void cdw_button_focus(CDW_BUTTON *button);
void cdw_button_unfocus(CDW_BUTTON *button);

void textarea_print_message(WINDOW *window, const char *message, int align);


CDW_DROPDOWN *cdw_dropdown_new(WINDOW *parent, int begin_y, int begin_x, int width, const char *item_labels[], int current_item_ind, int colors);
cdw_rv_t cdw_dropdown_destroy(CDW_DROPDOWN *dropdown);
void cdw_dropdown_display_current_item(CDW_DROPDOWN *dropdown);
void cdw_dropdown_display_current_item_reverse(CDW_DROPDOWN *dropdown, bool reverse);
cdw_rv_t cdw_dropdown_driver(CDW_DROPDOWN *dropdown, int *ind);
int cdw_dropdown_current_item_ind(CDW_DROPDOWN *dropdown);
void cdw_dropdown_set_current_item(CDW_DROPDOWN *dropdown, int ind);
void cdw_dropdown_focus(CDW_DROPDOWN *dropdown);
void cdw_dropdown_unfocus(CDW_DROPDOWN *dropdown);

#endif /* ifndef CDW_WIDGETS_H__ */
