/* cdw
 * Copyright (C) 2002 Varkonyi Balazs
 * Copyright (C) 2007 - 2009 Kamil Ignacak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <string.h>

#include "main.h"
#include "commands.h"
#include "processwin.h"
#include "gettext.h"
#include "utils_string.h"
#include "options.h"
#include "thread.h"
#include "cdw_disc.h"
#include "dvd_rw_format_interface.h"
#include "cdw_debug.h"

#include "config.h"



extern cdw_disc_t current_disc;
extern struct conf config;

/* was blanking of DVD+RW really performed? if yes, then it will be correct
   to put '100% done' message in processwin; otherwise it would be misleading */
extern bool dvd_rw_format_pipe_regexp_performed;

cdw_rv_t run_command_dvd_rw_format_blank(cdw_task_t *task)
{
	char *command = NULL;

	if (current_disc.disc_type == CDW_DVD_RW
		   || current_disc.disc_type == CDW_DVD_RW_SEQ
		   || current_disc.disc_type == CDW_DVD_RW_RES) {

		/* in case of blanking such kind of discs (DVD-RW) it is
		   important to know not only if this should be 'fast' or
		   'all', but also mode in which formatted disc should be:
		   'Restricted Overwrite' or 'Incremental Sequential' */
		if (task->blanking.format_mode == DVD_RW_FORMAT_SEQ) {
			cdw_sdm("task->blanking.format_mode == DVD_RW_FORMAT_SEQ\n");
			command = concat(DVD_RW_FORMAT,
				" -speed=", config.speed, " ",
				" -blank", config.blank_fast ? " " : "=full ",
				config.cdrw_device,
				(char *) NULL);
		} else if (task->blanking.format_mode == DVD_RW_FORMAT_RES) {
			cdw_sdm("task->blanking.format_mode == DVD_RW_FORMAT_RES\n");
			command = concat(DVD_RW_FORMAT,
				" -speed=", config.speed, " ",
				" -force", config.blank_fast ? " " : "=full ",
				config.cdrw_device,
				(char *) NULL);
		} else {
			cdw_assert(0, "task->blanking.format_mode is incorrect: %d\n", task->blanking.format_mode);
		}

	} else if (current_disc.disc_type == CDW_DVD_RWP) {
		command = concat(DVD_RW_FORMAT,
				" -speed=", config.speed, " ",
				/* " -force ", */
				config.cdrw_device,
				(char *) NULL);
	} else {
		return CDW_GEN_ERROR;
	}


	if (command == NULL) {
		return CDW_MEM_ERROR;
	}

	/* 2TRANS: this is title of process window - current action is blanking disc */
	processwin_create(_("Blank DVD"),
			  /* 2TRANS: this is message in process window,
			   cdw is performing blanking of DVD */
			  _("Blanking of DVD in progress..."), true);

	if (current_disc.disc_type == CDW_DVD_RW
		   || current_disc.disc_type == CDW_DVD_RW_SEQ
		   || current_disc.disc_type == CDW_DVD_RW_RES) {
		if (config.blank_fast) {
			/* 2TRANS: this is message in process window,
			   it says about second mode of blanking */
			processwin_display_text_info(2, _("Blanking in \'fast\' mode"));

		} else {
			/* 2TRANS: this is message in process window -
			   blanking a DVD is a time-consuming task */
			processwin_display_text_info(2, _("This process takes a long time"));
		}
	}

	cdw_sdm ("command is \"%s\"\n", command);
	//cdw_assert (0, "breaking in last moment\n");
	run_command(command, task);

	free(command);
	command = NULL;

	if (current_disc.disc_type == CDW_DVD_RWP
		   && dvd_rw_format_pipe_regexp_performed) {
		/* blanking dvd+rw is so fast that there are only a few values
		   of percentage displayed. 100% is not one of them. let's
		   fake it here */

		processwin_display_progress(100, 100, "");
	}

	/* 2TRANS: this is message in process window - program finished
	blanking disc, status unknown */
	processwin_destroy(_("Blanking of DVD finished"), true);

	return CDW_OK;
}

