/* cdw
 * Copyright (C) 2002 Varkonyi Balazs
 * Copyright (C) 2007 - 2009 Kamil Ignacak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include <sys/types.h>
#include <regex.h>

#include <ncursesw/ncurses.h>

#include "processwin.h"
#include "gettext.h"
#include "pipe_regexp.h"
#include "dvd_rw_format_pipe_regexp.h"
#include "thread.h" /* PIPE_BUFFER_SIZE */



/* extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1]; */
extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1];

bool dvd_rw_format_pipe_regexp_performed;

static regex_t *eregex1 = NULL;
static regmatch_t *ematches1 = NULL;


static void handle_dvd_rw_format_blanking(regex_t *eregex, regmatch_t *ematches);




/* regular expressions setup code */
void dvd_rw_format_stderr_regexp_prepare(void)
{
	dvd_rw_format_pipe_regexp_performed = false;

	int rv;

	/* "blanking 85.1%" */
	/* "formatting 85.1%" */

	/* currently cdw utilizes dvd+rw-format only to blank media;
	dvd+rw-format erases DVD-RW by normal blanking ("blanking 85.1%"
	is displayed); DVD+RW is erased by formatting ("formatting 85.1%" is
	displayed */

	/* don't put 'blanking' nor 'formatting' into regexp because
	dvd+rw-format puts '^H' bytes between it and percent value */
	eregex1 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(eregex1, "([0-9]+)([.]*)([0-9]*)%", REG_EXTENDED);
	ematches1 = (regmatch_t *) calloc(1, (eregex1->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, eregex1, 501);
	}

	return;
}





void dvd_rw_format_stderr_regexp_execute(void)
{
	/* debug message */
	/* fprintf(stderr, "%s(): stderr_pipe_buffer = \"%s\"\n", __func__, stderr_pipe_buffer); */

	int i = 0;
	/* remove funny characters from buffer, but don't operate on whole
	string - only few first chars are interesting */
	for (i = 0; i < 50 && i < PIPE_BUFFER_SIZE; i++) {
		if (!isprint(stderr_pipe_buffer[i])) {
			stderr_pipe_buffer[i] = ' ';
		}
	}
	/* fprintf(stderr, "%s(): stderr_pipe_buffer after cleaning = \"%s\"\n", __func__, stderr_pipe_buffer); */

	/* Case 1: "blanking 85.1%" */
	int rv1 = regexec(eregex1, stderr_pipe_buffer, eregex1->re_nsub + 1, ematches1, 0);
	if (rv1 == 0) {
		/* debug message */
		/* fprintf(stderr, "%s(): calling handle_dvd_rw_format_blanking() (stderr)\n", __func__); */

		handle_dvd_rw_format_blanking(eregex1, ematches1);
	}


	return;
}





void dvd_rw_format_stderr_regexp_destroy(void)
{
	regfree(eregex1);
	free(eregex1);

	free(ematches1);

	return;
}





void handle_dvd_rw_format_blanking(regex_t *regex, regmatch_t *matches)
{
	dvd_rw_format_pipe_regexp_performed = true;

	/* debug message */
	/* fprintf(stderr, "%s() called\n", __func__); */

	/* data to collect */
	int perc_a = 0, perc_b = 0;

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); ++i) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stderr_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		if (i == 0) { /* whole string, do nothing */
			/* debug message */
			/* fprintf(stderr, "i = %d, submatch = %s\n", i, submatch); */
		} else if (i == 1) { /* percentage - main part */
			/* fprintf(stderr, "i = %d, submatch = %s\n", i, submatch); */
			if (!strcmp(submatch, "??")) {
				perc_a = 0;
			} else {
				perc_a = atoi((char *) &submatch);
			}
		} else if (i == 2) { /* percentage - decimal point */
			/* debug message */
			/* fprintf(stderr, "i = %d, submatch = %s\n", i, submatch); */
		} else if (i == 3) { /* percentage - decimal part */
			/* debug message */
			/* fprintf(stderr, "i = %d, submatch = %s\n", i, submatch); */

			if (!strcmp(submatch, "??")) {
				perc_b = 0;
			} else {
				perc_b = atoi((char *) &submatch);
			}
		} else {
			;
		}
	}

	/* float percent = perc_a + 0.1 * perc_b; */
	/* NOTE: consider implementing
	conditional_processwin_display_progress_float(float done, long total, char *string)
	*/

	/* compute data */
	conditional_processwin_display_progress(perc_a, 100, "");

	/* debug messages */
	/* fprintf(stderr, "%s(): perc_a = %d\n", __func__, perc_a); */

	return;
}

