/* cdw
 * Copyright (C) 2002 Varkonyi Balazs
 * Copyright (C) 2007 - 2009 Kamil Ignacak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdlib.h>
#include <stdio.h>

#include "main.h"
#include "config.h"

#include "gettext.h"
#include "thread.h" /* run_command() */
#include "options.h" /* struct conf */
#include "processwin.h"
#include "cdw_ui.h"
#include "cdw_disc.h"
#include "commands.h"
#include "cdw_widgets.h"
#include "utils_string.h"
#include "color.h"
#include "dvd_rw_mediainfo_interface.h"


extern struct conf config;      /* variable holding cdw configuration */
extern cdw_disc_t current_disc; /* variable storing information about disc currently in drive */






/**
 * Get basic info about disc currently in drive
 *
 * Run dvd+rw-mediainfo to obtain some information about disc
 * that is currently in drive. The information is written to log file and will
 * be processed by regexp code.
 */
cdw_rv_t run_command_dvd_rw_mediainfo(cdw_task_t *task)
{
	char *command = concat(DVD_RW_MEDIAINFO, " ", config.cdrw_device, (char *) NULL);
	if (command == NULL) {
		/* 2TRANS: this is title of dialog window */
		dialogbox(_("Error"),
			  /* 2TRANS: this is message in dialog window */
			  _("An error occurred when attempting to read DVD info."),
			    DIALOG_OK, CDW_COLORS_ERROR);
		return CDW_MEM_ERROR;
	}

	current_disc.dvd_rw_mediainfo.blank_track_index = 0; /* reset track counter/index */

	/* 2TRANS: this is message in dialog window, displayed when
	program reads some meta-data from dvd */
	processwin_create(_("Load DVD info..."), "", false);

	run_command(command, task);

	processwin_destroy("", false);

	free(command);
	command = NULL;

	return CDW_OK;
}
