/* cdw
 * Copyright (C) 2002 Varkonyi Balazs
 * Copyright (C) 2007 - 2009 Kamil Ignacak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <regex.h>

#include <ncursesw/ncurses.h>

#include "processwin.h"
#include "options.h"
#include "thread.h" /* PIPE_BUFFER_SIZE */
#include "pipe_regexp.h"
#include "gettext.h"
#include "cdw_disc.h"
#include "cdw_debug.h"
#include "dvd_rw_mediainfo_pipe_regexp.h"


/* plus_16 comment:
 * somewhere below you can find comment with plus_16 in it, commenting
 * line, where mysterious value of 16 is added to other value. 16 was guessed
 * after analyzing outputs from k3b and dvd+rw-mediainfo, and was confirmed
 * after reading k3b sources. The value appears in sources of k3b in
 * k3b-1.0.4/libk3b/projects/datacd/k3bmsinfofetcher.cpp in definition of
 * K3bMsInfoFetcher::slotMediaDetectionFinished().
 *
 * For some reason the value is added to value of starting address of last
 * track on dvd disc. I'm sure that there is some good explanation for this,
 * I'm just too lazy to read T10 specs ;-)
 */


extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1];
/* extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1]; */



extern struct conf config;
extern cdw_disc_t current_disc;

extern WINDOW *processwin;

/* there is a special way of checking if DVD+RW is empty, even if
   "Disc status" line in dvd+rw-mediainfo says otherwise; this is a flag that
   is set to true if DVD+RW is REALLY empty, and its state should override
   value of "Disc status" */

static regex_t *regex1 = NULL, *regex3 = NULL, *regex4 = NULL, *regex5 = NULL;
static regex_t *regex6 = NULL, *regex7 = NULL, *regex8 = NULL, *regex9 = NULL;
static regmatch_t *matches1 = NULL, *matches3 = NULL, *matches4 = NULL;
static regmatch_t *matches5 = NULL, *matches6 = NULL, *matches7 = NULL;
static regmatch_t *matches8 = NULL, *matches9 = NULL;



static void handle_disc_status(regex_t *regex, regmatch_t *matches);
static void handle_dvd_write_speed(regex_t *regex, regmatch_t *matches);
static void handle_mounted_media(regex_t *regex, regmatch_t *matches);
/* valid but unused */
/* static void handle_dvd_toc(regex_t *regex, regmatch_t *matches); */
static void handle_track_start_address(regex_t *regex, regmatch_t *matches);
static void handle_next_writable_address(regex_t *regex, regmatch_t *matches);
static void handle_current_write_speed(regex_t *regex, regmatch_t *matches);
static void handle_read_capacity(regex_t *regex, regmatch_t *matches);

/* Legacy lead-out at:    2295104*2KB=4700372992 */


void dvd_rw_mediainfo_stdout_regexp_prepare(void)
{
	int rv;

	/*  "Write Speed #0:        4.0x1385=5540KB/s" */
	regex1 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex1, "Write Speed #([0-9]+):        ([0-9]+).([0-9]+)x", REG_EXTENDED);
	matches1 = (regmatch_t *) calloc(1, (regex1->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex1, 1);
	}

	/* "Disc status:           complete" (appendable/blank/other) */
	regex3 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex3, "Disc status:([ ]+)([a-zA-Z]+)", REG_EXTENDED);
	matches3 = (regmatch_t *) calloc(1, (regex3->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex3, 3);
	}

	/* "Mounted Media:         1Ah, DVD+RW" */
	/* "Mounted Media:         1Ah, DVD-RW Sequential" */
	regex4 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex4, "Mounted Media:([ ]+)([a-zA-Z0-9]+),([ ]+)([\-a-zA-Z +]*)", REG_EXTENDED);
	matches4 = (regmatch_t *) calloc(1, (regex4->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex4, 4);
	}

	/* " Track#2  :             14@145056" - information about DVD TOC,
	last number is start of track; track number may be also AA, whatever
	it means in DVD context */
	regex5 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex5, "Track#([0-9A]+)([ ]*):([ ]*)([0-9]*)@([0-9]+)", REG_EXTENDED);
	matches5 = (regmatch_t *) calloc(1, (regex5->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex5, 5);
	}

	/* "Track Start Address:   666512" - capture start addresses of DVD tracks*/
	regex6 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex6, "Track Start Address:([ ]*)([0-9]+)", REG_EXTENDED);
	matches6 = (regmatch_t *) calloc(1, (regex6->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex6, 6);
	}

	/* "Next Writable Address: 470672*2KB" start address for next session */
	regex7 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex7, "Next Writable Address:([ ]*)([0-9]+)", REG_EXTENDED);
	matches7 = (regmatch_t *) calloc(1, (regex7->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex7, 7);
	}

	/* "Current Write Speed:   16.0x1385=22160KB/s" */
	regex8 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex8, "Current Write Speed:([ ]*)([0-9]+)", REG_EXTENDED);
	matches8 = (regmatch_t *) calloc(1, (regex8->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex8, 8);
	}

	/* "READ CAPACITY:          0*2048=0" */
	/* read capacity for empty disc is 0 */
	regex9 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex9, "READ CAPACITY:([ ]+)([0-9]+)", REG_EXTENDED);
	matches9 = (regmatch_t *) calloc(1, (regex9->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler((char *)__func__, rv, regex9, 9);
	}

	return;
}





void dvd_rw_mediainfo_stdout_regexp_execute(void)
{

	/*  "Write Speed #0:        4.0x1385=5540KB/s" */
	int rv1 = regexec(regex1, stdout_pipe_buffer, regex1->re_nsub + 1, matches1, 0);
	if (rv1 == 0) {
		handle_dvd_write_speed(regex1, matches1);
	}

	/* "Disc status:           xxxxxxx" */
	int rv3 = regexec(regex3, stdout_pipe_buffer, regex3->re_nsub + 1, matches3, 0);
	if (rv3 == 0) {
		handle_disc_status(regex3, matches3);
	}

	/* "Mounted Media:         1Ah, DVD+RW" */
	int rv4 = regexec(regex4, stdout_pipe_buffer, regex4->re_nsub + 1, matches4, 0);
	if (rv4 == 0) {
		handle_mounted_media(regex4, matches4);
	}

	/* " Track#2  :             14@145056" */
	int rv5 = regexec(regex5, stdout_pipe_buffer, regex5->re_nsub + 1, matches5, 0);
	if (rv5 == 0) {
		/* valid but unused */
		/* handle_dvd_toc(regex5, matches5); */
	}

	/* " Track Start Address:   666512" */
	int rv6 = regexec(regex6, stdout_pipe_buffer, regex6->re_nsub + 1, matches6, 0);
	if (rv6 == 0) {
		handle_track_start_address(regex6, matches6);
	}

	/* "Next Writable Address: 470672*2KB" */
	int rv7 = regexec(regex7, stdout_pipe_buffer, regex7->re_nsub + 1, matches7, 0);
	if (rv7 == 0) {
		handle_next_writable_address(regex7, matches7);
	}

	/* "Current Write Speed:   16.0x1385=22160KB/s" */
	int rv8 = regexec(regex8, stdout_pipe_buffer, regex8->re_nsub + 1, matches8, 0);
	if (rv8 == 0) {
		handle_current_write_speed(regex8, matches8);
	}

	/* "READ CAPACITY:          0*2048=0" */
	int rv9 = regexec(regex9, stdout_pipe_buffer, regex9->re_nsub + 1, matches9, 0);
	if (rv9 == 0) {
		cdw_sdm("dispatching to handle_read_capacity()\n");
		handle_read_capacity(regex9, matches9);
	}

	return;
}





void dvd_rw_mediainfo_stdout_regexp_destroy(void)
{
	regfree(regex1);
	regfree(regex3);
	regfree(regex4);
	regfree(regex5);
	regfree(regex6);
	regfree(regex7);
	regfree(regex8);
	regfree(regex9);


	free(regex1);
	free(regex3);
	free(regex4);
	free(regex5);
	free(regex6);
	free(regex7);
	free(regex8);
	free(regex9);

	free(matches1);
	free(matches3);
	free(matches4);
	free(matches5);
	free(matches6);
	free(matches7);
	free(matches8);
	free(matches9);

	return;
}





/**
 * \brief Parse writing speeds reported by dvd+rw-mediainfo
 *
 * dvd+rw-mediainfo writes to stdout writing speeds in following format:
 * \verbatim
Write Speed #0:        16.0x1385=22160KB/s
Write Speed #1:        12.0x1385=16620KB/s
Write Speed #2:        10.0x1385=13850KB/s
Write Speed #3:        8.0x1385=11080KB/s
Write Speed #4:        4.0x1385=5540KB/s
\endverbatim
 *
 * All that needs to be done is to convert #0 (#1, #2...) into number,
 * use the number to index current_disc.dvd_rw_mediainfo.write_speed[] table,
 * and store writing speed in the table.
 */
void handle_dvd_write_speed(regex_t *regex, regmatch_t *matches)
{
	size_t speed_i = 0;

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); ++i) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* number of available speed */
			speed_i = atoi(submatch);

			/* this will work in both cases if dvd+rw-mediainfo
			   would list speed indexes from highest to lowest and
			   from lowest to highest */
			if (speed_i + 1 > current_disc.dvd_rw_mediainfo.n_speeds) {
				current_disc.dvd_rw_mediainfo.n_speeds = speed_i + 1; /* number of elements in table */
			}
		} else if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* speed value; write_speeds[] has limited size */
			if (speed_i < SPEED_TABLE_SIZE) {
				current_disc.dvd_rw_mediainfo.write_speeds[speed_i] = atoi(submatch);
			}
		} else if (i == 3) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			/* value after decimal point, not too interesting, since writing speeds are integers */
		}
	}

	return;
}





/**
 * \brief Handle string describing status of disc
 *
 * Handle string printed by dvd+rw-mediainfo that has form of:
 * "Disc status:           xxxxxxx"
 *
 * \verbatim
               1       2
"Disc status:([ ]+)([a-zA-Z]+)"
\endverbatim
 *
 * Disc status for DVD can be one of following:
 * "blank", "appendable", "complete", "other"
 */
void handle_disc_status(regex_t *regex, regmatch_t *matches)
{
	char status_string[11];

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		} else if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
			strncpy(status_string, submatch, len + 1);
		}
		/* only 2 submatches defined */
	}

	/* do string comparisons here, so higher-level code doesn't
	   have to do this */

	/* all this may be overwritten (as CDW_DISC_EMPTY_YES)
	   for DVD-RW if read capacity of disc == 0 */

	if (!strcmp(status_string, "blank")) {
		cdw_sdm("setting disc status to empty\n");
		current_disc.empty = CDW_DISC_EMPTY_YES;
		current_disc.appendable = CDW_DISC_APPENDABLE_YES; /* NOTE: should we do this here? */

	} else if (!strcmp(status_string, "appendable")) {
		cdw_sdm("setting disc status to appendable\n");
		current_disc.appendable = CDW_DISC_APPENDABLE_YES;

	} else if (!strcmp(status_string, "complete")) {
		cdw_sdm("setting disc status to complete\n");
		current_disc.appendable = CDW_DISC_APPENDABLE_CLOSED;

	} else if (!strcmp(status_string, "other")) {
		cdw_sdm("setting disc status to other / unknown\n");
		current_disc.appendable = CDW_DISC_APPENDABLE_UNKNOWN;

	} else {
		cdw_sdm("setting disc status to error\n");
		current_disc.appendable = CDW_DISC_APPENDABLE_ERROR;
	}

	return;
}





/**
 * \brief Extract information about media type
 *
 * Extract information (media type) from dvd+rw-mediainfo string which looks
 * like this:
 *
 * \verbatim
" Mounted Media:         1Bh, DVD+R"

                 1         2          3        4
"Mounted Media:([ ]+)([a-zA-Z0-9]+),([ ]+)([\-a-zA-Z +]*)"
\endverbatim
 *
 * This function sets current_disc.disc_type
 */
void handle_mounted_media(regex_t *regex, regmatch_t *matches)
{
	assert (regex->re_nsub == 4);

	size_t len = matches[4].rm_eo - matches[4].rm_so;
	char submatch[PIPE_BUFFER_SIZE];
	strncpy(submatch, (char *) &stdout_pipe_buffer[matches[4].rm_so], len);
	submatch[len] = '\0';

	/* debug message */
	cdw_sdm("submatch%d (media type): \"%s\"\n", 4, submatch);

	/* convert string value into int value so that
		higher level code doesn't have to compare strings */
	if (!strncmp(submatch, "DVD-R Sequential", 16)) {
		current_disc.disc_type = CDW_DVD_R_SEQ;

	} else if (!strncmp(submatch, "DVD-R Restricted", 16)) {
		current_disc.disc_type = CDW_DVD_R_RES;

	} else if (!strcmp(submatch, "DVD-R")) {
		current_disc.disc_type = CDW_DVD_R;

	} else if (!strncmp(submatch, "DVD-RW Sequential", 17)) {
		current_disc.disc_type = CDW_DVD_RW_SEQ;

	} else if (!strncmp(submatch, "DVD-RW Restricted", 17)) {
		current_disc.disc_type = CDW_DVD_RW_RES;

	} else if (!strncmp(submatch, "DVD-RW", 6)) {
		current_disc.disc_type = CDW_DVD_RW;

	} else if (!strcmp(submatch, "DVD+R")) {
		current_disc.disc_type = CDW_DVD_RP;

	} else if (!strcmp(submatch, "DVD+RW")) {
		current_disc.disc_type = CDW_DVD_RWP;

	} else if (!strcmp(submatch, "DVD-ROM")) {
		current_disc.disc_type = CDW_DVD_ROM;

	} else {
		current_disc.disc_type = CDW_DISC_UNKNOWN;
	}

	return;
}





/**
 * \brief Handle dvd+rw-mediainfo strings with track start indexes
 *
 * dvd+rw-mediainfo output looks like this:
 * \verbatim
Track Start Address:   470672*2KB
\endverbatim
 * and it is repeated for every track. Note that there is no track number,
 * so we use current_disc.dvd_rw_mediainfo.blank_track_index to index
 * current_disc.dvd_rw_mediainfo.track_address[] and point where to store
 * successive track start addresses.
 *
 * The function has similar effect to handle_dvd_toc(), but it also captures
 * starting sector of blank area, which is used as part of '-C' parameter
 * of growisofs when writing multisession discs.
 *
 * handle_track_start_address() does not capture AA track.
 * handle_dvd_toc() does it.
 */
void handle_track_start_address(regex_t *regex, regmatch_t *matches)
{
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); ++i) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* track start address */
			int tsa = atoi(submatch);
			/* first increment and then use - this is not a mistake:
			 blank_track_index is initialized to zero, so it will
			 be equal to 1 when storing first tsa, it will be
			 blank_track_index when storing tsa of BLANK track,
			 and will be (blank_track_index - 1) when we will be
			 storing tsa of last written (real) track */
			current_disc.dvd_rw_mediainfo.blank_track_index++;
			current_disc.dvd_rw_mediainfo.track_address[current_disc.dvd_rw_mediainfo.blank_track_index] = tsa;

		} else {
			;
		}
	}

	return;
}





/**
 * Capture address of first sector of next track/session
 *
 * dvd+rw-mediainfo prints following information about "empty" track, the
 * track that follows last track written to disc:
 * \verbatim
STDOUT READ TRACK INFORMATION[#4]:
 Track State:           blank
 Track Start Address:   470672*2KB
 Next Writable Address: 470672*2KB
 Free Blocks:           1824432*2KB
 Track Size:            1824432*2KB
\endverbatim
 *
 * The string that we must capture and process is
 * "Next Writable Address: 470672*2KB". This string is written after all data
 * is processed by handle_track_start_address().
 * Therefore we can use:
 *
 * (current_disc.dvd_rw_mediainfo.blank_track_index - 1)
 * to get
 * last session start from current_disc.dvd_rw_mediainfo.track_address[]
 *
 * (current_disc.dvd_rw_mediainfo.blank_track_index)
 * to get
 * next (blank) session start from current_disc.dvd_rw_mediainfo.track_address[]
 *
 * However "next (blank) session start" is not quite the same as "Next Writable
 * Address":
 * It may seem that values of "Track Start Address" and
 * "Next Writable Address" are the same, but dvd+rw-mediainfo uses two
 * different expressions to calculate the values. I use
 * "Next Writable Address" (nwa) instead of '"Track Start Address" of
 * blank track' as value of next_sess_start.
 */
void handle_next_writable_address(regex_t *regex, regmatch_t *matches)
{
	long int nwa = 0;

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); ++i) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* next writable address */
			nwa = atol(submatch);
		} else {
			;
		}
	}


	if (current_disc.dvd_rw_mediainfo.blank_track_index == 1) {
		/* blank track is first track on disc, so both last_sess_start
		   and next_sess_start should be 0 */
		current_disc.last_sess_start = 0;
		current_disc.next_sess_start = 0;
	} else {
		/* see comment at the beginning of file, starting with plus_16 */
		current_disc.last_sess_start = current_disc.dvd_rw_mediainfo.track_address[current_disc.dvd_rw_mediainfo.blank_track_index - 1] + 16;
		current_disc.next_sess_start = nwa;
	}

	/* debug messages */
	/*
	fprintf(stderr, "%s(): current_disc.last_sess_start - 16 = %ld\n", __func__, current_disc.last_sess_start - 16);
	fprintf(stderr, "%s(): current_disc.last_sess_start = %ld\n", __func__, current_disc.last_sess_start);
	fprintf(stderr, "%s(): current_disc.next_sess_start = %ld\n", __func__, current_disc.next_sess_start);
	*/
	return;
}




void handle_current_write_speed(regex_t *regex, regmatch_t *matches)
{
	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); ++i) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 2) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* "Current Write Speed */
			current_disc.drive.default_speed = atoi(submatch);
		} else {
			;
		}
	}

	/* debug message */
	/* fprintf(stderr, "%s(): current_disc.drive.default_speed = %d\n", __func__, current_disc.drive.default_speed); */

	return;
}





/**
 *\verbatim
"READ CAPACITY:          0*2048=0"

                 1      2
"READ CAPACITY:([ ]+)([0-9]+)"
\endverbatim
 */
void handle_read_capacity(regex_t *regex, regmatch_t *matches)
{
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); ++i) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 2) {
			/* debug message */
			cdw_sdm ("submatch%d: \"%s\"\n", i, submatch);

			/* since regexp code matched this submatch as
			   ([0-9]+), then value '0' possibly returned by
			   atoi() is not a sign of incorrectly formed
			   string in its argument, but it means that the
			   arguments translates to zero */
			current_disc.dvd_rw_mediainfo.read_capacity = atoi(submatch);
		} else {
			;
		}
	}

	cdw_sdm("read_capacity = %d\n", (int) current_disc.dvd_rw_mediainfo.read_capacity);

	return;
}




#if 0
/* handle_dvd_toc() is valid, but unused */

/**
 * Handle output of dvd+rw-mediainfo when it prints TOC table
 *
 * TOC printed by dvd+rw-mediainfo looks like this:
 * \verbatim
FABRICATED TOC:
 Track#1  :             14@0
 Track#2  :             14@145056
 Track#3  :             14@309712
 Track#4  :             14@470672
 Track#AA :             14@601376
\endverbatim
 *
 * Note that there is a track number, so we can use it to index writing speed
 * table. Note also that tracks are numbered from 1.
 *
 * The function has similar effect to handle_track_start_address() but it does
 * not capture start of blank area. However it does capture start of AA area.
 */
void handle_dvd_toc(regex_t *regex, regmatch_t *matches)
{
	char track_index[3];
	char track_start_address[10];
	int i_track;

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); ++i) {
		int len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';
		/* debug message */
		/* fprintf(stderr, "----%s(): matching submatch: \"%s\"\n", __func__, submatch); */
		if (i == 1) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* track index, 1-based */
			strncpy(track_index, submatch, len + 1);

			/* track number reported by dvd+rw-mediainfo can be
			either a number or "AA" string */
			if (!strncmp(track_index, "AA", 2)) {
				i_track = 99;
			} else {
				i_track = atoi(track_index);
			}
		} else if (i == 5) {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */

			/* start of i-th track */
			strncpy(track_start_address, submatch, len + 1);

			/* track numbers reported by dvd+rw-mediainfo are
			1-based, we will keep it that way; this will be also
			compatible with cdio */
			current_disc.dvd_rw_mediainfo.track_address[i_track] = atol(track_start_address);
		} else {
			;
		}
	}

	return;
}

#endif

