#ifndef _FILELIST_H
#define _FILELIST_H


#include <ncursesw/ncurses.h> /* bool */

#include "main.h" /* cdw_rv_t */
#include "cdw_file_manager.h" /* cdw_file_t */



/* public */
cdw_rv_t filelist_append_file(const cdw_file_t cdw_file);
cdw_rv_t filelist_remove_file(size_t i);

size_t filelist_length(void);
cdw_rv_t filelist_clean(void);

/* you could use filelist_length() instead, but this is faster
   when calling it for long, non-empty lists */
bool filelist_is_empty(void);

cdw_rv_t filelist_to_file(void);
long long filelist_dirsize(void);





/* private, declarations for testing in filelist_tests.c only */

/** \brief Type of variables that are elements of list
 *
 * Should not be used outside of filelist.c and filelist_tests.c */
typedef struct _filelist_item_t filelist_item_t;
struct _filelist_item_t {
	filelist_item_t *prev;
	filelist_item_t *next;
	cdw_file_t cdw_file;
};

cdw_rv_t filelist_init(const cdw_file_t cdw_file);
filelist_item_t *filelist_ith_item(size_t i);
char *filelist_ith_fullpath(size_t i);
filelist_item_t *filelist_new_item(const cdw_file_t cdw_file);
filelist_item_t *filelist_last_item(void);
cdw_rv_t filelist_append_file_non_unique(const cdw_file_t cdw_file);
bool filelist_is_member(const cdw_file_t cdw_file);

#endif /* _FILELIST_H */
