/* cdw
 * Copyright (C) 2002 Varkonyi Balazs
 * Copyright (C) 2007 - 2009 Kamil Ignacak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <unistd.h>

#include <fcntl.h>

#include <ncursesw/ncurses.h>

#include "config.h"
#include "main.h"

#include "cdw_ui_main.h"
#include "cdw_widgets.h"
#include "options.h"
#include "gettext.h"
#include "color.h"
#include "help.h"


extern WINDOW *main_cdw_win;

/** \brief Row of app window, in which first line of tip will be displayed */
#define TIP_ROW_1 (LINES - 3)
/** \brief Row of app window, in which second line of tip will be displayed */
#define TIP_ROW_2 (LINES - 2)


/**
 * \brief Show window with list of hotkeys available in main application window
 */
void show_help_main(void)
{
	int nlines = 20;
	int ncols = 60;
	int begin_y = (LINES - 20) / 2;
	int begin_x = (COLS - 60 ) / 2;

	int col = 2;

	WINDOW *window = newwin(nlines, ncols, begin_y, begin_x);
	wbkgd(window, COLOR_PAIR(CDW_COLORS_DIALOG));
	werase(window);
	keypad(window, TRUE);

	box(window, 0, 0);
	/* 2TRANS: this is title of small help window displaying hotkeys */
	nice_box(window, _("Help"),
		 /* 2TRANS: this is message at the bottom of help window */
		 _("Press ESCAPE to close window"));

	mvwprintw(window, 2, col, "%s",
		  /* 2TRANS: this is label explaining contnent of
		     help window */
		  _("Hotkeys available in main window:"));

	mvwprintw(window, 4, col, "%s",
		  /* 2TRANS: this is hotkey label: "show this help window";
		     keep position of '-' */
		  _("F1, H, ?  - Show this help"));
	mvwprintw(window, 5, col, "%s",
		  /* 2TRANS: this is hotkey label: "show window, in which
		     user can enter/edit CD label (volume ID)";
		     keep position of '-' */
		  _("F2        - Show disc label window"));
	mvwprintw(window, 6, col, "%s",
		  /* 2TRANS: this is hotkey label: "show window, in which
		     user can edit additional parameters of 'cdrecord'";
		     keep position of '-' */
		  _("F3        - Edit 'cdrecord' parameters"));
	mvwprintw(window, 7, col, "%s",
		  /* 2TRANS: this is hotkey label: "show dialog with
		     selector of size of disc, to which user want to
		     write data;  keep position of '-' */
		  _("F4        - Select target disc size"));
	mvwprintw(window, 8, col, "%s",
		  /* 2TRANS: this is hotkey label: "show window displaying
		     messages from last task"; keep position of '-'  */
		  _("F5, L     - Show log of last operation"));
	mvwprintw(window, 9, col, "%s",
		  /* 2TRANS: this is hotkey label: "show window with some
		     disc meta-data"; keep position of '-' */
		  _("F6        - Show information about disc"));
	mvwprintw(window, 10, col, "%s",
		  /* 2TRANS: this is hotkey label: "show window displaying
		     cdw license"; keep position of '-' */
		  _("F10       - Show license of this program"));
	mvwprintw(window, 11, col, "%s",
		  /* 2TRANS: this is hotkey label: "eject tray of your
		     CD/DVD drive"; keep position of '-' */
		  _("E         - Eject drive tray"));
	/* mvwprintw(window, 10, 2, "%s", _("C   - Disk catalog"));
	 * mvwprintw(window, 11, 2, "%s", _("A   - Add disk into Disk catalog")); */
	mvwprintw(window, 12, col, "%s",
		  /* 2TRANS: this is hotkey label, it means "copy tracks
		     from audio CD to your hard disc";
		     keep position of '-' */
		  _("G         - Grab audio CD"));
	mvwprintw(window, 13, col, "%s",
		  /* 2TRANS: this is hotkey label: "burn ISO9660 file
		     to optical disc"; keep position of '-' */
		  _("I         - Write ISO image to disc"));
	mvwprintw(window, 14, col, "%s",
		  /* 2TRANS: this is hotkey label: "burn files that you
		     selected to optical disc"; keep position of '-' */
		  _("F         - Write selected files to disc"));
	mvwprintw(window, 15, col, "%s",
		  /* 2TRANS: this is hotkey label: "read some information
		     about disc that is in drive and display it in app
		     window"; keep position of '-' */
		  _("R         - Refresh information about disc"));
	mvwprintw(window, 17, col, "%s",
		  /* 2TRANS: this is hotkey label, it means
		    "close application"; keep position of '-' */
		  _("Q         - Quit"));
	wrefresh(window);

	int c = 'a';
	while (c != CDW_ESCAPE) {
		c = wgetch(window);
	}

	delwin(window);
	window = (WINDOW *) NULL;

	main_win_refresh_part(nlines, ncols, begin_y, begin_x);

	return;
}




void show_help_cddb(void)
{
    int c = 0;
    WINDOW *help=newwin(20,60,(LINES-20)/2,(COLS-60)/2);
    wbkgd(help, COLOR_PAIR(CDW_COLORS_DIALOG));
    werase(help);
    box(help,0,0);
    keypad(help, TRUE);
    mvwaddch(help, 0, 2, ACS_RTEE);
    mvwprintw(help, 0, 3, " %s ", _("Help"));
    waddch(help, ACS_LTEE);
    mvwprintw(help, 19, 2, "( %s )", _("Press ESC to close window"));
    mvwprintw(help, 2, 2, "%s", _("->     - Step into catalogized files window"));
    mvwprintw(help, 3, 2, "%s", _("<-     - Step into menu window"));
    mvwprintw(help, 4, 2, "%s", _("ENTER  - Change category or disk or directory"));
    mvwprintw(help, 5, 2, "%s", _("BSPACE"));
    mvwprintw(help, 6, 2, "%s", _("/      - Move one level up"));
    mvwprintw(help, 7, 2, "%s", _("DEL    - Delete category or disk"));
    mvwprintw(help, 8, 2, "%s", _("A      - Add disk into catalog"));
    mvwprintw(help, 9, 2, "%s", _("S      - Show all disks"));
    mvwprintw(help,10, 2, "%s", _("Q      - Quit Disk catalog"));

    wrefresh(help);
    while (c != CDW_ESCAPE) {
	c=wgetch(help);
    }
    delwin(help);
}





void display_tooltip(int item)
{
	wattrset(main_cdw_win, A_BOLD | COLOR_PAIR(CDW_COLORS_TOOLTIPS));
	switch(item) {
		case UI_MENU_ADD_FILES:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string; it says "this menu item opens
			file selector dialog */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3,_("Select the files that you want to write to optical disc                "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string; this tooltip lists allowed
			keys used in file selector dialog */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("UP/DOWN arrows to move, Space to select file, Escape to finish        "));
			break;
		case UI_MENU_DELETE_FILES:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string; it says "this menu item will
			move you to list of selected files where you can remove items from the list */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Deselect files from list                                               "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string; this tooltip describes allowed
			keys used on 'selected files' list */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("UP/DOWN arrows to move, Delete to deselect files, Escape to finish     "));
			break;
		case UI_MENU_WRITE_DATA_FILES_TO_IMAGE:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Create iso image using selected files                                  "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Create on your hdd iso file suitable for writing to CD or DVD          "));
			break;
		case UI_MENU_COPY_DATA_DISC_TO_IMAGE:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Copy content of CD disc to image file                                  "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Create copy of your CD disc on your hard disc                          "));
			break;
		case UI_MENU_WRITE_DATA_FILES_TO_OPT_DISC:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Write selected files to optical disc                                   "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Write files that are visible in right-hand panel to CD or DVD          "));
			break;
		case UI_MENU_WRITE_IMAGE_TO_OPT_DISC:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Write ISO image file to optical disc                                   "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Write ISO image file specified in Configuration to CD or DVD           "));
			break;
		case UI_MENU_RIP_AUDIO_DISC:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Copy audio CD to your hard drive                                       "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Copy audio tracks from audio CD to separate files on hard disc         "));
			break;
		case UI_MENU_BLANK_DISC:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Blank rewritable optical disc                                          "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Erase content of CD-RW or DVD+/-RW - you can choose method             "));
			break;
		case UI_MENU_EDIT_OPTIONS:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Edit configuration                                                     "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Edit cdw configuration like speed of recording or volume label         "));
			break;
		case UI_MENU_EXIT_CDW:
			/* 2TRANS: this is short tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, _("Close program                                                          "));
			/* 2TRANS: this is long tooltip displayed at the bottom of main cdw window; pad with blanks at the end
			to keep length of string, translation must be no longer than whole string */
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, _("Close this program, return to your command line session                "));
			break;
		default:
			mvwprintw(main_cdw_win, TIP_ROW_1, 3, "                                                                         ");
			mvwprintw(main_cdw_win, TIP_ROW_2, 3, "                                                                         ");
			break;
	}
	wattrset(main_cdw_win, A_NORMAL | COLOR_PAIR(CDW_COLORS_MAIN));

	return;
}



