/* cdw
 * Copyright (C) 2002 Varkonyi Balazs
 * Copyright (C) 2007 - 2009 Kamil Ignacak
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "thread.h"
#include "pipe_regexp.h"
#include "tasks.h"
#include "md5sum_interface.h"
#include "cdw_debug.h"



extern char stdout_pipe_buffer[PIPE_BUFFER_SIZE + 1];
/* extern char stderr_pipe_buffer[PIPE_BUFFER_SIZE + 1]; */


extern int matched_result; /* how many times did we catch result of md5sum? */
extern char md5sum_digest_cd[];
extern char md5sum_digest_image[];



/* functions used for stdout output handling */
static void handle_md5sum_output(regex_t *regex, regmatch_t *matches);


static regex_t *regex1 = NULL;
static regmatch_t *matches1 = NULL;




/**
 * \brief Allocate memory for regexp data, compile regular expressions (stdout version)
 *
 * Allocate memory for all variables of type regex_t and regmatch_t that
 * are used in processing of data in stdout_pipe_buffer[].
 *
 * Compile all regular expressions that are used in processing of data in
 * stdout_pipe_buffer[].
 *
 * Call regcomp_error_handler() on errors.
 */
void md5sum_stdout_regexp_prepare(void)
{
	int rv;
	/* dea059d48b26fc29602e4f130d27614c  file_name */
	regex1 = (regex_t *) calloc(1, sizeof(regex_t));
	rv = regcomp(regex1, "([a-zA-Z0-9]+)", REG_EXTENDED);
	matches1 = (regmatch_t *) calloc(1, (regex1->re_nsub + 1) * sizeof(regmatch_t));
	if (rv) {
		regcomp_error_handler(__func__, rv, regex1, 1);
	}

	return;

}





/**
 * \brief Match data in stdout_pipe_buffer[] agains regular expressions, call handler function
 *
 * This function calls regexec() for all regular expressions to compare them
 * against data in stdout_pipe_buffer[]. If there will be any match, the
 * function will call proper haldler function to further process data from
 * stdout_pipe_buffer[]
 */
void md5sum_stdout_regexp_execute(void)
{
	stdout_pipe_buffer[PIPE_BUFFER_SIZE - 1] = '\0';

	/* dea059d48b26fc29602e4f130d27614c  file_name */
	int rv1 = regexec(regex1, stdout_pipe_buffer, regex1->re_nsub + 1, matches1, 0);
	if (rv1 == 0) {
		handle_md5sum_output(regex1, matches1);
	}

	return;
}





/**
 * \brief Free all memory used by regular expression variables (stdout version)
 *
 * Call regfree() and free() all variables of type regex_t and regmatch_t that
 * were used in processing of data in stdout_pipe_buffer[].
 *
 * See also cdrecord_stderr_regexp_destroy()
 */
void md5sum_stdout_regexp_destroy(void)
{
	regfree(regex1);

	free(regex1);

	free(matches1);

	return;
}





/**
 * \brief Function stores 32-char digest in md5sum_digestX
 */
void handle_md5sum_output(regex_t *regex, regmatch_t *matches)
{
	/* debug message */
	/* fprintf(stderr, "%s() called\n", __func__); */

	/* collect data */
	unsigned int i;
	for (i = 0; i < (regex->re_nsub + 1); i++) {
		size_t len = matches[i].rm_eo - matches[i].rm_so;
		char submatch[PIPE_BUFFER_SIZE];
		strncpy(submatch, (char *) &stdout_pipe_buffer[matches[i].rm_so], len);
		submatch[len] = '\0';

		/* debug message */
		/* fprintf(stderr, "%s(): current submatch: \"%s\"\n", __func__, submatch); */

		if (i == 1) {
			matched_result++;
			cdw_sdm("submatch%d: \"%s\" (catch %d)\n", i, submatch, matched_result);
			if (matched_result == 1) {
				strncpy(md5sum_digest_cd, submatch, len + 1);
			} else if (matched_result == 2) {
				strncpy(md5sum_digest_image, submatch, len + 1);
			}
		} else  {
			/* debug message */
			/* fprintf(stderr, "%s(): submatch%d: \"%s\"\n", __func__, i, submatch); */
		}


	}

	return;
}

