#ifndef TASKS_H__
#define TASKS_H__

#include <ncursesw/ncurses.h> /* bool */


/* mode of writing files to a CD/DVD disc */
typedef enum {
	CDW_WRITE_GUARD = -1,
	CDW_WRITE_NONE = 0,
	CDW_WRITE_START_MULTI,      /**< \brief Start writing to disc in mode allowing further appending of data (don't close disc after writing) */
	CDW_WRITE_CREATE_SINGLE,    /**< \brief Start writing to disc in mode NOT allowing further appending of data (close disc after writing)*/
	CDW_WRITE_CONTINUE_MULTI,   /**< \brief Continue writing to disc in mode allowing further appending of data (don't close disc after writing) */
	CDW_WRITE_WRITE_FINAL,      /**< \brief Write to disc in mode NOT allowing further appending of data (close disc after writing) */
	CDW_WRITE_ERROR
} cdw_write_mode_t;



typedef struct  {
	enum {
		TASK_NONE,            /**< no task currently assigned */

		TASK_BURN,            /**< write to optical disc (imag or files) */
		TASK_READ,            /**< read content of optical disc */
		TASK_ERASE,           /**< erase optical disc */
		TASK_CLONE,           /**< clone one optical disc to another optical disc */

		TASK_CREATE_IMAGE,    /**< create iso image on hdd, using files */

		TASK_CHECK_MD5SUM,    /**< check md5sum of specified object */

		TASK_MEDIAINFO,       /**< check media information of dvd disc */
		TASK_MSINFO,          /**< get multisession information of cd disc */
		TASK_ATIP             /**< get atip information of cd disc */
	} id; /** \brief Identifier of given task */

	enum {
		TOOL_NONE,

		TOOL_CDRECORD,
		TOOL_MKISOFS,
		TOOL_MD5SUM,
		TOOL_GROWISOFS,
		TOOL_DVD_RW_FORMAT,
		TOOL_DVD_RW_MEDIAINFO,
		TOOL_LIBCDIO
	} tool; /** \brief Tool used to perform given task */

	enum {
		OBJ_NONE,

		OBJ_ISO_IMAGE,
		OBJ_OPT_DISC,
		OBJ_DATA,
		OBJ_AUDIO,

		OBJ_HD,
		OBJ_FD,

		OBJ_DATA_CD,
		OBJ_AUDIO_CD,
		OBJ_DVD
	} source, target; /** \brief Source and target objects of given task */


	enum {
		TOOL_STATUS_NONE = 0x8000,
		TOOL_STATUS_OK = 0x0000,
		TOOL_STATUS_CDRECORD_CANNOT_OPEN_DEVICE = 0x0001,
		TOOL_STATUS_CDRECORD_NO_SUCH_FILE_OR_DIR = 0x0002,
		TOOL_STATUS_LIBCDIO_SETUP_FAIL = 0x0004,
		TOOL_STATUS_LIBCDIO_READ_FAIL = 0x0008,
		TOOL_STATUS_GROWISOFS_NO_SPACE_LEFT = 0x0010,
		TOOL_STATUS_GROWISOFS_MEDIA_NOT_APPENDABLE = 0x0020,
		TOOL_STATUS_MKISOFS_THE_SAME_JOLIET_NAME = 0x0040,
		TOOL_STATUS_MKISOFS_JOLIET_TREE_SORT = 0x0080,
		TOOL_STATUS_MKISOFS_CANT_FIND_BOOT_IMAGE = 0x0100,
		TOOL_STATUS_MKISOFS_NO_SUCH_FILE_OR_DIR = 0x0200,
		TOOL_STATUS_MKISOFS_BAD_OPTION = 0x0400,
		TOOL_STATUS_MD5SUM_WRONG_TRACK = 0x0800,
		TOOL_STATUS_MKISOFS_INODE_NUMBER_OVERFLOW = 0x1000,
		TOOL_STATUS_MKISOFS_NO_SPACE_LEFT = 0x2000
	} tool_status;

	struct {
		/* if after attempting to blank of CD-RW value of
		   blanking_time is still zero, then blanking failed */
		size_t blanking_time;

		enum {
			DVD_RW_FORMAT_NONE,

			DVD_RW_FORMAT_SEQ,
			DVD_RW_FORMAT_RES
		} format_mode;
	} blanking;

	bool success;
	bool verify_write;

	cdw_write_mode_t write_mode;

} cdw_task_t; /** \brief Data structure describing task that is currently
                         performed or is about to be performed */

void task_set_state(cdw_task_t *task, int id, int tool, int source, int target);
void task_reset_state(cdw_task_t *task);
void task_check_status(cdw_task_t *task);


#endif /* TASKS_H__ */
